import { FunctionContext } from "./tryCatchFinally";
export interface ITryCatchFinallyHook<TParentContext = FunctionContext, TrtReturnContext = {}> {
    onTry(ctx: TParentContext & FunctionContext): void | {
        context?: TrtReturnContext;
        onCatch?(ctx: FunctionContext & TParentContext & TrtReturnContext): void;
        onFinally?(ctx: FunctionContext & TParentContext & TrtReturnContext): void;
        /**
         * If true, the hook will be executed after all other hooks
         */
        lastInQueue?: boolean;
    };
}
export type HookContextOf<THook> = THook extends ITryCatchFinallyHook<infer T1, infer T2> ? T1 & T2 : never;
export type DecoratorArgsOf<THookContext> = THookContext extends {
    args?: infer TArgs extends {};
} ? TArgs : {};
export declare class TryCatchFinallyHooksBuilder<FullHookContext extends FunctionContext = FunctionContext> {
    private hooks;
    add<TryReturnContext = {}>(onTry: ITryCatchFinallyHook<FullHookContext, TryReturnContext>['onTry']): TryCatchFinallyHooksBuilder<FullHookContext & TryReturnContext>;
    add<THookContext extends {}, TryReturnContext extends {} = {}>(hook: ITryCatchFinallyHook<FullHookContext & THookContext, TryReturnContext>): TryCatchFinallyHooksBuilder<THookContext & FullHookContext & TryReturnContext>;
    static createHook<TryHookContext, TryReturnContext = {}>(onTry: ITryCatchFinallyHook<TryHookContext & FunctionContext, TryReturnContext>['onTry']): ITryCatchFinallyHook<TryHookContext & FunctionContext, TryReturnContext>;
    asFunctionWrapper(args?: DecoratorArgsOf<FullHookContext>): <TFunc extends (...args: any[]) => any>(func: TFunc) => TFunc;
    asDecorator(args?: DecoratorArgsOf<FullHookContext>): MethodDecorator;
    createDecorator(): (args: DecoratorArgsOf<FullHookContext>) => MethodDecorator;
    protected beforeHooksTry(ctx: FullHookContext): {
        onCatch?(): void;
        onFinally?(): void;
    };
    protected afterHooksTry?(ctx: FullHookContext): {
        onCatch?(): void;
        onFinally?(): void;
    };
    asScope<TResult>(args: DecoratorArgsOf<FullHookContext>, scope: () => TResult): TResult;
    private forEachHook;
    private _currentContext;
    /**
     * Only safe to use in sync functions or in async functions before any awaited code.
     * Otherwise, the context may be changed by another async function - in that case use callStack hook instead
     */
    get current(): (FullHookContext & FunctionContext) | undefined;
}
export type ContextOf<TTryCatchFinallyHooksBuilder extends TryCatchFinallyHooksBuilder<any>> = TTryCatchFinallyHooksBuilder extends TryCatchFinallyHooksBuilder<infer T1> ? T1 : never;
//# sourceMappingURL=TryCatchFinallyHooks.d.ts.map