"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TryCatchFinallyHooksBuilder = void 0;
const tryCatchFinally_1 = require("./tryCatchFinally");
class TryCatchFinallyHooksBuilder {
    constructor() {
        this.hooks = [];
        this._currentContext = undefined;
    }
    add(hook) {
        if (hook instanceof Function)
            return this.add(TryCatchFinallyHooksBuilder.createHook(hook));
        this.hooks.push(hook);
        return this;
    }
    static createHook(onTry) {
        return {
            onTry
        };
    }
    asFunctionWrapper(args) {
        var _a;
        const _this = this;
        const beforeHooksTry = this.beforeHooksTry.bind(this);
        const afterHooksTry = (_a = this.afterHooksTry) === null || _a === void 0 ? void 0 : _a.bind(this);
        return (func) => {
            return (0, tryCatchFinally_1.createTryCatchFinally)(func, {
                onTry(funcCtx) {
                    const ctx = funcCtx;
                    if (args)
                        ctx.args = args;
                    const bht = beforeHooksTry(ctx);
                    const hooksRes = _this.forEachHook(hook => hook.onTry(ctx));
                    for (const hookRes of [...hooksRes]) {
                        if (hookRes && hookRes.lastInQueue) {
                            const [itemToMove] = hooksRes.splice(hooksRes.indexOf(hookRes), 1);
                            hooksRes.push(itemToMove);
                        }
                    }
                    return {
                        onFinally(_ctx) {
                            var _a;
                            for (const hr of hooksRes) {
                                if (hr && hr.onFinally)
                                    hr.onFinally(_ctx);
                            }
                            (_a = bht.onFinally) === null || _a === void 0 ? void 0 : _a.call(bht);
                        },
                        onCatch(_ctx) {
                            var _a;
                            for (const hr of hooksRes) {
                                if (hr && hr.onCatch)
                                    hr.onCatch(_ctx);
                            }
                            (_a = bht.onCatch) === null || _a === void 0 ? void 0 : _a.call(bht);
                        }
                    };
                },
            });
        };
    }
    asDecorator(args) {
        return (target, propertyKey, descriptor) => {
            descriptor.value = this.asFunctionWrapper(args)(descriptor.value);
        };
    }
    createDecorator() {
        return args => this.asDecorator(args);
    }
    beforeHooksTry(ctx) {
        const _this = this;
        const prevContext = _this._currentContext;
        this._currentContext = ctx;
        return {
            onFinally() {
                _this._currentContext = prevContext;
            },
        };
    }
    asScope(args, scope) {
        return this.asFunctionWrapper(args)(scope)();
    }
    forEachHook(fn) {
        return this.hooks.map(h => fn(h));
    }
    /**
     * Only safe to use in sync functions or in async functions before any awaited code.
     * Otherwise, the context may be changed by another async function - in that case use callStack hook instead
     */
    get current() {
        return this._currentContext;
    }
}
exports.TryCatchFinallyHooksBuilder = TryCatchFinallyHooksBuilder;
